/*
 * $Id: rand.h,v 1.1.1.1 2002/11/03 11:08:24 kenta Exp $
 *
 * Copyright 2002 Kenta Cho. All rights reserved.
 */

/**
 * Make random number function.
 *
 * @version $Revision: 1.1.1.1 $
 */
unsigned int nextRandInt(unsigned int *v);
