//
//  smart_ptr.hpp
//
//  For compatibility, this header includes the headers for the four "classic"
//  smart pointer class templates.
//
//  Copyright (c) 1998-2002 boost.org
//
//  Permission to copy, use, modify, sell and distribute this software
//  is granted provided this copyright notice appears in all copies.
//  This software is provided "as is" without express or implied
//  warranty, and with no claim as to its suitability for any purpose.
//
//  http://www.boost.org/libs/smart_ptr/smart_ptr.htm
//

#include <boost/scoped_ptr.hpp> 
#include <boost/scoped_array.hpp>
#include <boost/shared_ptr.hpp>
#include <boost/shared_array.hpp>
